@@ -1,5 +1,5 @@
 // Copyright (C) 2015 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 
 
@@ -22,6 +22,8 @@
 #include <QMutex>
 #include <QCborMap>
 #include <QtCore/qhash.h>
+#include <QtCore/qqueue.h>
+#include <QtCore/qnumeric.h>
 #include <QtCore/private/qobject_p.h>
 
 QT_BEGIN_NAMESPACE
@@ -40,13 +42,25 @@ public:
 private:
     bool loadMeta();
     void loadPlugin();
+    void updateState(QTextToSpeech::State newState);
+    void disconnectSynthesizeFunctor();
     static void loadPluginMetadata(QMultiHash<QString, QCborMap> &list);
     QTextToSpeech *q_ptr;
     QTextToSpeechPlugin *m_plugin = nullptr;
-    QTextToSpeechEngine *m_engine = nullptr;
+    std::unique_ptr<QTextToSpeechEngine> m_engine = nullptr;
     QString m_providerName;
     QCborMap m_metaData;
     static QMutex m_mutex;
+    QQueue<QString> m_pendingUtterances;
+    QTextToSpeech::State m_state = QTextToSpeech::Error;
+    QMetaObject::Connection m_synthesizeConnection;
+    QtPrivate::QSlotObjectBase *m_slotObject = nullptr;
+
+    qsizetype m_utteranceCounter = 0;
+    qsizetype m_currentUtterance = 0;
+    double m_storedPitch = qQNaN();
+    double m_storedVolume = qQNaN();
+    double m_storedRate = qQNaN();
 };
 
 QT_END_NAMESPACE
